import { world, ItemStack, BlockPermutation, MolangVariableMap } from '@minecraft/server';
import { ModalFormData, ActionFormData } from "@minecraft/server-ui";
import { leftBlockLocation, rightBlockLocation, colors, frontBlockLocation, backBlockLocation, airBlocks } from '../util/globalVariables';
import { addOrReplaceItem, decrementItemInHand, getDirectionByPlayer, handleSitOnFurniture, isCreative, itemApplyDamage, spawnItemSilkTouch } from '../util/utils';

function extractColor(blockId) {
	const colorPattern = new RegExp(colors.join("|"), "i");
	const match = blockId.match(colorPattern);
	return match ? match[0] : 'black';
}
world.beforeEvents.worldInitialize.subscribe((e) => {
	e.blockComponentRegistry.registerCustomComponent("dev_modf:office_desk", new ModernOfficeDesk());
	e.blockComponentRegistry.registerCustomComponent("dev_modf:office_desk_cabinet", new ModernOfficeDeskCabinet());
	e.blockComponentRegistry.registerCustomComponent("dev_modf:sit_on_chair", new ModernSitOnChair());
	e.blockComponentRegistry.registerCustomComponent("dev_modf:trash_can", new ModernTrashCan());
	e.blockComponentRegistry.registerCustomComponent("dev_modf:ceiling_fan", new ModernCeilingFan());
	e.blockComponentRegistry.registerCustomComponent("dev_modf:ceiling_fan_timer", new ModernCeilingFanTimer());
	e.blockComponentRegistry.registerCustomComponent("dev_modf:printer", new ModernPrinter());
	e.blockComponentRegistry.registerCustomComponent("dev_modf:place_bed", new ModernPlaceBed());
	e.blockComponentRegistry.registerCustomComponent("dev_modf:bed_silk_touch", new ModernBedSilkTouch());
	e.blockComponentRegistry.registerCustomComponent("dev_modf:sofa", new ModernSofa());
	e.blockComponentRegistry.registerCustomComponent("dev_modf:sit_on_toilet", new ModernSitOnToilet());
	e.blockComponentRegistry.registerCustomComponent("dev_modf:mirror", new ModernMirror());
	e.blockComponentRegistry.registerCustomComponent("dev_modf:fridge", new ModernFridge());
	e.blockComponentRegistry.registerCustomComponent("dev_modf:fridge_top", new ModernFridgeTop());
	e.blockComponentRegistry.registerCustomComponent("dev_modf:sink", new ModernSink());
	e.blockComponentRegistry.registerCustomComponent("dev_modf:bucket", new ModernBucket());
	e.blockComponentRegistry.registerCustomComponent("dev_modf:furniture", new ModernFurnitureTable());
	e.blockComponentRegistry.registerCustomComponent("dev_modf:furniture_color", new ModernFurnitureTableColor());
	e.itemComponentRegistry.registerCustomComponent("dev_modf:paint_blocks", new ModernPaintBlocks());
	e.itemComponentRegistry.registerCustomComponent("dev_modf:bucket_place", new ModernBucketPlace());
});
class ModernFurnitureTableColor {
	onPlayerInteract(e) {
		const { player, block, dimension } = e;
		const color = extractColor(block.typeId);
		const item = player.getComponent("inventory").container.getItem(player.selectedSlotIndex);
		const allowedDyes = ["minecraft:black_dye","minecraft:blue_dye","minecraft:brown_dye","minecraft:cyan_dye","minecraft:gray_dye","minecraft:green_dye","minecraft:light_blue_dye","minecraft:light_gray_dye","minecraft:lime_dye","minecraft:magenta_dye","minecraft:orange_dye","minecraft:pink_dye","minecraft:purple_dye","minecraft:red_dye","minecraft:white_dye","minecraft:yellow_dye"];
		if (!item) return;
		const itemColor = extractColor(item.typeId);
		if (allowedDyes.includes(item.typeId) && color !== itemColor) {
			block.setPermutation(BlockPermutation.resolve(block.typeId.replace(color, itemColor), { "minecraft:cardinal_direction": block.permutation.getState("minecraft:cardinal_direction")}));
			dimension.playSound("copper.wax.on", block.center());
			decrementItemInHand(player);
			return;
		}
		if (item.typeId !== "minecraft:iron_ingot") {
			player.onScreenDisplay.setActionBar({ translate: 'form.dev_modf.player_use_iron_ingot' });
			return;
		}
		const tableUI = new ActionFormData()
		.title("DecoFlex Furniture")
		.button(``, `textures/ui/dev_modf/${color}/bed_${color}`)
		.button(``, `textures/ui/dev_modf/${color}/ceiling_fan_${color}`)
		.button(``, `textures/ui/dev_modf/${color}/ceiling_lamp_${color}`)
		.button(``, `textures/ui/dev_modf/${color}/ceiling_lamp2_${color}`)
		.button(``, `textures/ui/dev_modf/${color}/computer_${color}`)
		.button(``, `textures/ui/dev_modf/${color}/file_${color}`)
		.button(``, `textures/ui/dev_modf/${color}/fridge_${color}`)
		.button(``, `textures/ui/dev_modf/${color}/glass_table_${color}`)
		.button(``, `textures/ui/dev_modf/${color}/lamp_${color}`)
		.button(``, `textures/ui/dev_modf/${color}/mirror_${color}`)
		.button(``, `textures/ui/dev_modf/${color}/office_chair_${color}`)
		.button(``, `textures/ui/dev_modf/${color}/office_chair2_${color}`)
		.button(``, `textures/ui/dev_modf/${color}/office_desk_${color}`)
		.button(``, `textures/ui/dev_modf/${color}/office_desk_cabinet_${color}`)
		.button(``, `textures/ui/dev_modf/${color}/phone_${color}`)
		.button(``, `textures/ui/dev_modf/${color}/pot_0_${color}`)
		.button(``, `textures/ui/dev_modf/${color}/pot_1_${color}`)
		.button(``, `textures/ui/dev_modf/${color}/printer_${color}`)
		.button(``, `textures/ui/dev_modf/${color}/sink_${color}`)
		.button(``, `textures/ui/dev_modf/${color}/sofa_${color}`)
		.button(``, `textures/ui/dev_modf/${color}/toilet_${color}`)
		.button(``, `textures/ui/dev_modf/${color}/trash_can_${color}`)
		.button(``, `textures/ui/dev_modf/${color}/tv_${color}`);
		tableUI.show(player).then((r) => {
			if (r.canceled) return;
			let itemName = "minecraft:iron_block";
			switch (r.selection) {
				case 0: itemName = `dev_modf:bed_2x2_${color}`;break;
				case 1: itemName = `dev_modf:ceiling_fan_${color}`;break;
				case 2: itemName = `dev_modf:ceiling_lamp_${color}`;break;
				case 3: itemName = `dev_modf:ceiling_lamp2_${color}`;break;
				case 4: itemName = `dev_modf:computer_${color}`;break;
				case 5: itemName = `dev_modf:file_${color}`;break;
				case 6: itemName = `dev_modf:fridge_${color}`;break;
				case 7: itemName = `dev_modf:glass_table_${color}`;break;
				case 8: itemName = `dev_modf:lamp_${color}`;break;
				case 9: itemName = `dev_modf:mirror_${color}`;break;
				case 10: itemName = `dev_modf:office_chair_${color}`;break;
				case 11: itemName = `dev_modf:office_chair2_${color}`;break;
				case 12: itemName = `dev_modf:office_desk_${color}`;break;
				case 13: itemName = `dev_modf:office_desk_cabinet_${color}`;break;
				case 14: itemName = `dev_modf:phone_${color}`;break;
				case 15: itemName = `dev_modf:pot_0_${color}`;break;
				case 16: itemName = `dev_modf:pot_1_${color}`;break;
				case 17: itemName = `dev_modf:printer_${color}`;break;
				case 18: itemName = `dev_modf:sink_${color}`;break;
				case 19: itemName = `dev_modf:sofa_${color}`;break;
				case 20: itemName = `dev_modf:toilet_${color}`;break;
				case 21: itemName = `dev_modf:trash_can_${color}`;break;
				case 22: itemName = `dev_modf:tv_${color}`;break;
			}
			const form = new ModalFormData()
			.title({ translate: 'form.dev_modf.furniture_table.select_quantity' })
			.slider({ translate: 'form.dev_modf.furniture_table.quantity' }, 1, item.amount, 1, 1)
			form.show(player).then((r) => {
				if (r.canceled) return;
				const [quantity] = r.formValues;
				dimension.spawnItem(new ItemStack(itemName, quantity), block.center());
				decrementItemInHand(player, false, quantity);
			});
		});
	}
}
class ModernFurnitureTable {
	onPlayerInteract(e) {
		const { player, block, dimension } = e;
		const allowedDyes = ["minecraft:black_dye","minecraft:blue_dye","minecraft:brown_dye","minecraft:cyan_dye","minecraft:gray_dye","minecraft:green_dye","minecraft:light_blue_dye","minecraft:light_gray_dye","minecraft:lime_dye","minecraft:magenta_dye","minecraft:orange_dye","minecraft:pink_dye","minecraft:purple_dye","minecraft:red_dye","minecraft:white_dye","minecraft:yellow_dye"];
		const item = player.getComponent("inventory").container.getItem(player.selectedSlotIndex);
		if (allowedDyes.includes(item?.typeId)) {
			block.setPermutation(BlockPermutation.resolve(`${block.typeId}_${extractColor(item.typeId)}`, { "minecraft:cardinal_direction": block.permutation.getState("minecraft:cardinal_direction")}));
			dimension.playSound("copper.wax.on", block.center());
			decrementItemInHand(player);
		} else {
			player.onScreenDisplay.setActionBar({ translate: 'form.dev_modf.player_use_dye' });
		}
	}
}
class ModernBucketPlace {
	onUseOn(e) {
		const { source, itemStack, blockFace, block } = e;
		if (blockFace !== "Up" || !airBlocks.includes(block.above().typeId)) return;
		const direction = getDirectionByPlayer(source);
		const fillState = itemStack.getComponent("durability").damage;
		const color = extractColor(itemStack.typeId);
		block.above().setPermutation(BlockPermutation.resolve(`dev_modf:bucket_${color}_block`, {"minecraft:cardinal_direction": direction,"dev_modf:fill_state": fillState,}));
		block.dimension.playSound("dig.netherite", block.above().center());
		decrementItemInHand(source);
	}
}
class ModernBucket {
	onPlayerDestroy(e) {
		const { player, block, dimension, destroyedBlockPermutation } = e;
		if (isCreative(player)) return;
		const paintState = destroyedBlockPermutation.getState("dev_modf:fill_state");
		const spawnItem = paintState < 4 ? `dev_modf:bucket_${extractColor(destroyedBlockPermutation.type.id)}` : "minecraft:bucket";
		const item = new ItemStack(spawnItem, 1);
		const durability = item.getComponent("durability");
		if (durability && paintState < 4) durability.damage = paintState;
		dimension.spawnItem(item, block.center());
	}
	onPlayerInteract(e) {
		const { player, block, dimension } = e;
		const inventory = player.getComponent("inventory").container;
		const item = inventory.getItem(player.selectedSlotIndex);
		const paintState = block.permutation.getState("dev_modf:fill_state");
		const allowedDyes = ["minecraft:black_dye","minecraft:blue_dye","minecraft:brown_dye","minecraft:cyan_dye","minecraft:gray_dye","minecraft:green_dye","minecraft:light_blue_dye","minecraft:light_gray_dye","minecraft:lime_dye","minecraft:magenta_dye","minecraft:orange_dye","minecraft:pink_dye","minecraft:purple_dye","minecraft:red_dye","minecraft:white_dye","minecraft:yellow_dye"];
		const allowedBrushes = ['dev_modf:brush_default','dev_modf:brush_black','dev_modf:brush_blue','dev_modf:brush_brown','dev_modf:brush_cyan','dev_modf:brush_gray','dev_modf:brush_green','dev_modf:brush_light_blue','dev_modf:brush_light_gray','dev_modf:brush_lime','dev_modf:brush_magenta','dev_modf:brush_orange','dev_modf:brush_pink','dev_modf:brush_purple','dev_modf:brush_red','dev_modf:brush_white','dev_modf:brush_yellow'];
		const blockColor = extractColor(block.typeId);
		const itemColor = extractColor(item.typeId);
		if (allowedDyes.includes(item?.typeId) && blockColor !== itemColor) {
			block.setPermutation(BlockPermutation.resolve(`dev_modf:bucket_${extractColor(item.typeId)}_block`, {"minecraft:cardinal_direction": block.permutation.getState("minecraft:cardinal_direction")}));
			dimension.playSound("copper.wax.on", block.center());
			decrementItemInHand(player);
		}
		if (allowedBrushes.includes(item?.typeId) && paintState < 4 && blockColor !== itemColor) {
			const newBrush = new ItemStack(`dev_modf:brush_${blockColor}`, 1);
			const durability = item.getComponent("durability");
			if (durability) newBrush.getComponent("durability").damage = durability.damage;
			inventory.setItem(player.selectedSlotIndex, newBrush);
			dimension.playSound("copper.wax.on", block.center());
			block.setPermutation(block.permutation.withState("dev_modf:fill_state", paintState + 1));
		}
	}
}
class ModernPaintBlocks {
	onUse(e) {
		const { itemStack, source } = e;
		const direction = getDirectionByPlayer(source);
		const inventory = source.getComponent("inventory").container;
		const durabilityComponent = itemStack.getComponent("durability");
		let damageItem = durabilityComponent.damage;
		const block = source.dimension.getBlock(source.location);
		const width = 9, height = 5, depth = 7;
		const colorMap = {
			"black": {r: 0.18, g: 0.18, b: 0.21},
			"blue": {r: 0.27, g: 0.35, b: 0.74},
			"brown": {r: 0.53, g: 0.33, b: 0.19},
			"cyan": {r: 0.07, g: 0.62, b: 0.62},
			"gray": {r: 0.38, g: 0.44, b: 0.45},
			"green": {r: 0.34, g: 0.46, b: 0.08},
			"light_blue": {r: 0.16, g: 0.63, b: 0.84},
			"light_gray": {r: 0.58, g: 0.58, b: 0.55},
			"lime": {r: 0.48, g: 0.78, b: 0.09},
			"magenta": {r: 0.74, g: 0.24, b: 0.71},
			"orange": {r: 1, g: 0.51, b: 0.09},
			"pink": {r: 0.93, g: 0.47, b: 0.61},
			"purple": {r: 0.48, g: 0.16, b: 0.67},
			"red": {r: 0.71, g: 0.19, b: 0.16},
			"white": {r: 0.9, g: 0.92, b: 0.92},
			"yellow": {r: 1, g: 0.81, b: 0.16}
		}
		const particleColor = colorMap[extractColor(itemStack.typeId)];
		const particle = new MolangVariableMap();
		particle.setFloat('variable.paint_color_r', particleColor.r);
		particle.setFloat('variable.paint_color_g', particleColor.g);
		particle.setFloat('variable.paint_color_b', particleColor.b);
		particle.setFloat('variable.paint_color_a', 1);
		source.dimension.spawnParticle("dev_modf:paint_particle", source.location, particle);
		const applyDamage = (component) => {
			if (damageItem >= component.maxDurability) {
				inventory.setItem(source.selectedSlotIndex, undefined);
				source.dimension.playSound("random.break", source.location);
				return true;
			} else {
				damageItem += 1;
				return false;
			}
		};
		const canBePainted = (targetBlock, newColor) => {
			const colorBlock = extractColor(targetBlock.typeId);
			return targetBlock.hasTag("dev_modf.can_be_painted") && colorBlock !== newColor;
		};
		for (let y = 0; y < height; y++) {
			for (let x = 0; x < width; x++) {
				for (let z = 0; z < depth; z++) {
					let offsetX = x - Math.floor(width / 2);
					let offsetZ = z;
					if (direction === 'south') offsetZ = -z;
					else if (direction === 'east') [offsetX, offsetZ] = [-z, x - Math.floor(width / 2)];
					else if (direction === 'west') [offsetX, offsetZ] = [z, x - Math.floor(width / 2)];
					let targetBlock = block.offset({ x: offsetX, y: y, z: offsetZ });
					const newColor = extractColor(itemStack.typeId);
					if (canBePainted(targetBlock, newColor)) {
						targetBlock.setPermutation(BlockPermutation.resolve(targetBlock.typeId.replace(extractColor(targetBlock.typeId), newColor), targetBlock.permutation.getAllStates()));
						if (!isCreative(source) && applyDamage(durabilityComponent)) return;
					}
				}
			}
		}
		if (!isCreative(source)) {
			durabilityComponent.damage = damageItem;
			inventory.setItem(source.selectedSlotIndex, itemStack);
		}
	}
}
class ModernSink {
	onPlayerInteract(e) {
		const { block, dimension, player } = e;
		const item = player.getComponent("inventory").container.getItem(player.selectedSlotIndex);
		const waterLevel = block.permutation.getState("dev_modf:water_level");
		const itemActions = {
			"minecraft:water_bucket": {condition: () => waterLevel < 2, newLevel: waterLevel + 1, sounds: ["bucket.empty_water"], resultItem: "minecraft:bucket"},
			"minecraft:bucket": {condition: () => waterLevel > 0,newLevel: waterLevel - 1, sounds: ["bucket.fill_water"], resultItem: "minecraft:water_bucket"},
			"minecraft:glass_bottle": {condition: () => waterLevel > 0, newLevel: waterLevel - (Math.random() < 0.1 ? 1 : 0), sounds: ["bottle.dragonbreath", "bucket.fill_water"], resultItem: "minecraft:potion"}
		};
		if (item && itemActions[item.typeId]) {
			const { condition, newLevel, sounds, resultItem } = itemActions[item.typeId];
			if (condition()) {
				block.setPermutation(block.permutation.withState("dev_modf:water_level", newLevel));
				sounds.forEach(sound => dimension.playSound(sound, block.center()));
				if (resultItem) addOrReplaceItem(player, resultItem, 1);
			}
		} else {
			dimension.spawnParticle("dev_modf:water_sink", block.center());
		}
	}
}
class ModernFridgeTop {
	onPlayerDestroy(e) {
		const { block, dimension } = e;
		dimension.runCommandAsync(`setblock ${block.x} ${block.y - 1} ${block.z} air destroy`);
	}
}
class ModernFridge {
	beforeOnPlayerPlace(e) {
		const { block, dimension } = e;
		const topBlock = block.above();
		if (airBlocks.includes(topBlock.typeId)) {
			topBlock.setType("dev_modf:fridge_top");
			dimension.spawnEntity("dev_modf:container", {x: block.x + 0.5, y: block.y + 1, z: block.z + 0.5}).nameTag = "dev_modf.block.fridge";
		} else e.cancel = true;
	}
}
class ModernMirror {
	beforeOnPlayerPlace(e) {
		const { block, permutationToPlace } = e;
		const topBlock = block.above();
		airBlocks.includes(topBlock.typeId) ? topBlock.setPermutation(BlockPermutation.resolve("dev_modf:mirror_top", {"minecraft:cardinal_direction": permutationToPlace.getState("minecraft:cardinal_direction")})) : e.cancel = true;
	}
}
class ModernSitOnToilet {
	onPlayerInteract(e) {
		handleSitOnFurniture(e, 'dev_modf:chair_entity', 0.5);
	}
}
function UpdateSofa(block) {
	const permutation = block.permutation;
	const direction = permutation.getState("minecraft:cardinal_direction");
	const tag = `${block.typeId}_${direction}`;
	const rightBlock = block.offset(rightBlockLocation[direction]);
	const leftBlock = block.offset(leftBlockLocation[direction]);
	block.setPermutation(permutation.withState("dev_modf:r", rightBlock.hasTag(tag)).withState("dev_modf:l", leftBlock.hasTag(tag)));
}
function TestSofa(block, permutation) {
	let id = block.typeId;
	let direction = permutation.getState("minecraft:cardinal_direction");
	if (id !== "minecraft:air") UpdateSofa(block);
	else id = permutation.type.id;
	const tag = `${id}_${direction}`;
	const rightBlock = block.offset(rightBlockLocation[direction]);
	const leftBlock = block.offset(leftBlockLocation[direction]);
	[rightBlock, leftBlock].forEach(neighbor => { if (neighbor.hasTag(tag)) UpdateSofa(neighbor)});
}
class ModernSofa {
	onPlace(e) {
		TestSofa(e.block, e.block.permutation)
	}
	onPlayerDestroy(e) {
		TestSofa(e.block, e.destroyedBlockPermutation);
	}
}
class ModernCeilingFanTimer {
	onTick(e) {
		const { block } = e;
		block.setPermutation(block.permutation.withState("dev_modf:on", false));
		block.setPermutation(block.permutation.withState("dev_modf:timer_off", false));
	}
}
class ModernBedSilkTouch {
	onPlayerDestroy(e) {
		const color = extractColor(e.destroyedBlockPermutation.type.id);
		spawnItemSilkTouch(e, `dev_modf:bed_2x2_${color}`);
	}
}
class ModernPlaceBed {
	beforeOnPlayerPlace(e) {
		const { player, block, dimension, permutationToPlace } = e;
		const direction = getDirectionByPlayer(player);
		const rightBlock = block.offset(rightBlockLocation[direction]);
		const backBlock = block.offset(backBlockLocation[direction]);
		const rightBackBlock = rightBlock.offset(backBlockLocation[direction]);
		if ([rightBlock.typeId, backBlock.typeId, rightBackBlock.typeId].every(typeId => airBlocks.includes(typeId))) {
			const color = extractColor(permutationToPlace.type.id);
			block.setPermutation(BlockPermutation.resolve(`dev_modf:bed_2x2_${color}_00`, {"minecraft:cardinal_direction": direction}));
			rightBlock.setPermutation(BlockPermutation.resolve(`dev_modf:bed_2x2_${color}_01`, {"minecraft:cardinal_direction": direction}));
			backBlock.setPermutation(BlockPermutation.resolve(`dev_modf:bed_2x2_${color}_10`, {"minecraft:cardinal_direction": direction}));
			rightBackBlock.setPermutation(BlockPermutation.resolve(`dev_modf:bed_2x2_${color}_11`, {"minecraft:cardinal_direction": direction}));
			decrementItemInHand(player);
			dimension.playSound("dig.wood", block.center());
		}
		e.cancel = true;
	}
}
class ModernPrinter {
	onPlayerInteract(e) {
		const { player } = e;
		const inventory = player.getComponent("inventory").container;
		const item = inventory.getItem(player.selectedSlotIndex);
		const allowedItems = ['minecraft:name_tag', 'minecraft:paper', 'minecraft:book', 'minecraft:writable_book', 'minecraft:enchanted_book'];
		if (allowedItems.includes(item?.typeId)) {
			const form = new ModalFormData()
			.title({ translate: 'form.dev_modf.printer.rename_item' })
			.textField({ translate: 'form.dev_modf.printer.new_name' }, { translate: 'form.dev_modf.printer.write_new_name' });
			form.show(player).then((response) => {
				if (response.canceled) return;
				const [newItemName] = response.formValues;
				if (newItemName === '') return;
				item.nameTag = newItemName;
				inventory.setItem(player.selectedSlotIndex, item);
			});
		} else player.onScreenDisplay.setActionBar({ translate: 'form.dev_modf.printer.allowed_items' });
	}
}
class ModernCeilingFan {
	onPlayerInteract(e) {
		const { block, dimension } = e;
		const permutation = block.permutation;
		const onState = permutation.getState("dev_modf:on");
		const offTimer = permutation.getState("dev_modf:timer_off");
		const ceilingFanEntity = dimension.getEntitiesAtBlockLocation(block.location).find(entity => entity.typeId === 'dev_modf:ceiling_fan_entity');
		if (!onState && !offTimer) {
			block.setPermutation(permutation.withState("dev_modf:on", true));
			if (!ceilingFanEntity) dimension.spawnEntity("dev_modf:ceiling_fan_entity", block.center()).triggerEvent(`dev_modf:${extractColor(block.typeId)}`);
		} else {
			block.setPermutation(permutation.withState("dev_modf:timer_off", true));
			if (ceilingFanEntity) ceilingFanEntity.triggerEvent("dev_modf:turn_off");
		}
	}
}
class ModernTrashCan {
	beforeOnPlayerPlace(e) {
		const entity = e.dimension.spawnEntity("dev_modf:container", e.block.center());
		entity.triggerEvent("dev_modf:trash_can");
		entity.nameTag = "dev_modf.block.trash_can";
	}
	onPlayerInteract(e) {
		const { player, block, dimension } = e;
		const trashCanEntity = dimension.getEntitiesAtBlockLocation(block.location).find(entity => entity.typeId === 'dev_modf:container');
		if (trashCanEntity) {
			const inventory = trashCanEntity.getComponent("inventory").container;
			if (inventory.emptySlotsCount === inventory.size) return;
			const form = new ActionFormData()
			.title({ translate: 'form.dev_modf.trash_can.title' })
			.body({ translate: 'form.dev_modf.trash_can.body' })
			.button({ translate: 'form.dev_modf.trash_can.button_yes' })
			.button({ translate: 'form.dev_modf.trash_can.button_no' });
			form.show(player).then((response) => {
				if (response.canceled) return;
				if (response.selection === 0) {
					trashCanEntity.remove();
					const entity = dimension.spawnEntity("dev_modf:container", block.center());
					entity.triggerEvent("dev_modf:trash_can");
					entity.nameTag = "dev_modf.block.trash_can";
				}
			});
		}
	}
}
class ModernSitOnChair {
	onPlayerInteract(e) {
		handleSitOnFurniture(e, 'dev_modf:chair_entity', 0.4);
	}
}
function UpdateModernOfficeDesk(block) {
	const permutation = block.permutation;
	const direction = permutation.getState("minecraft:cardinal_direction");
	const tag = `dev_modf.office_desk_${extractColor(block.typeId)}_${direction}`;
	const rightBlock = block.offset(rightBlockLocation[direction]);
	const leftBlock = block.offset(leftBlockLocation[direction]);
	block.setPermutation(permutation.withState("dev_modf:r",rightBlock.hasTag(tag)).withState("dev_modf:l",leftBlock.hasTag(tag)));
}
function TestModernOfficeDesk(block, permutation) {
	let id = block.typeId;
	let direction = permutation.getState("minecraft:cardinal_direction");
	if (id !== "minecraft:air") UpdateModernOfficeDesk(block);
	else id = permutation.type.id;
	const tag = `dev_modf.office_desk_${extractColor(id)}_${direction}`;
	const rightBlock = block.offset(rightBlockLocation[direction]);
	const leftBlock = block.offset(leftBlockLocation[direction]);
	if (rightBlock.hasTag(tag)) UpdateModernOfficeDesk(rightBlock);
	if (leftBlock.hasTag(tag)) UpdateModernOfficeDesk(leftBlock);
}
class ModernOfficeDesk {
	onPlace(e) {
		TestModernOfficeDesk(e.block, e.block.permutation);
	}
	onPlayerDestroy(e) {
		TestModernOfficeDesk(e.block, e.destroyedBlockPermutation);
	}
}
class ModernOfficeDeskCabinet {
	beforeOnPlayerPlace(e) {
		e.dimension.spawnEntity("dev_modf:container", e.block.center()).nameTag = "dev_modf.block.desk_cabinet";
	}
}